#include "myshapes.h"
#include "randomizer.h" 
#include <gl\gl.h>
#include <gl\glu.h>
#include <gl\glut.h>
#include <assert.h>

#define SLICES 32
#define STACKS 16

cCheerleader::cCheerleader()
{
/* The molecules 4 cheerleader code is a primitive demo that looks
like a gun turret. You can replace this with your own code. */
//First fix the motion fields.
	setRotation(0.0, 0.0, 0.0, 1.0); /* Start non-rotated, ready to
		turn around z axis. */
	setPosition(0.0, 0.0, 0.0); 
	_animatespinrate = 45.0; //_animatespinrate is rotation rate in degrees per second.
	_minangle = -30.0;
	_maxangle = 30.0;
//Now build the geometry
	cPolyhedron  *parm_twirler, *ptorso;
//	cPolyhedron *parmplain, *phead; //Later I may want to use these other vars.
//	cAssemblyComposite *pbaton;
//	cPolyhedron *pballA, *pballB, *pstick;
	/* We want "this" to  be composite whose children are these
	cPolyhedron: ptorso, phead, parmplain, parm_twirler, parmplain,
	plus the cAssemblyComposite pbaton.
	pbaton has the cPolyedron children pballA, pballB, pstick. 
	Alternately, you could "glue" the baton to parm_twirler by assembling
	these into a cAssemblyComposite parm_twirler_and_baton.  
	**Let's assume the cheerleader stands on the origin, its body axis along
	the z axis, and its face in the x direction.  We put the parm_twirler
	halfway up the body facing along a 45 degree line between the 
	x and the y axis.
	**We make the polyhedra by using the constructor
		cPolyhedron(int gluttype = SPHERE, Real radius = 1.0,
		int slices = 32, int stacks = 16, Real edge = sqrt(4.0/3.0),
		Real tuberadius = 0.3,	Real bottomradius = 1.0,
		Real topradius = 1.0 , Real height = 3.0);
	**We glue assemblies and polyhedra together using 
		cAssemblyComposite::add(cAssembly *pa);
	**We adjust the polyhedra and composite assemblies positions using the
	  cAssembly methods
		void applyRotation(Real rdegrees, Real rx, Real ry, Real rz);
		void setScale(cVector scalevector){_scale = scalevector;}
		void setPosition(const cVector &newposition){_position = newposition;
	*/
//Create and add the ptorso.
	ptorso = new cPolyhedron(cPolyhedron::CYLINDER, /* Note that you must put
			put cPolyhedron:: in front of gluttypes like SPHERE, TORUS, etc. */
		1.0, SLICES, STACKS, 1.0, 1.0, //radius, slices, stacks, edge, tuberadius
		3.0, 2.0, 6.0); //Bottom radius, top radius, height.
	ptorso->setColor(cVector(0.8, 0.3, 0.1));
	ptorso->setUsesTexture(false);
	add(ptorso);
//Create and add the parm_twirler.
	parm_twirler = new cPolyhedron(cPolyhedron::CYLINDER, /* Note that you must put
			put cPolyhedron:: in front of gluttypes like SPHERE, TORUS, etc. */
		1.0, 6, STACKS, 1.0, 1.0, //radius, slices, stacks, edge, tuberadius
		0.75, 0.5, 4.0); //Bottom radius, top radius, height.
	parm_twirler->setPosition(sqrt(2.5)-0.2, sqrt(2.5)-0.2 , 3.0);
		/* Note that the position of a cylinder is the
		center of its base circle. We  move parm_twirler  halfway up the
		torso to a position above the 45 degree line between x and y axis.
		The radius there is 2.5, and if x and y are equal, we get sqrt of
		this.  I reduce this a bit, by 0.2, just to make sure there's no gap.
			The order in which I call setPosition in the constructor
		doesn't interact with when I call applyRotation.  I don't use the
		_position field until the draw method, and there I call glTranslate
		first so that it happens last.*/
	parm_twirler->applyRotation(45, 0.0, 0.0, 1.0);
		/* Rotate it around the z axis towards y to lie on the line y=x. 
		Remember that the FIRST rotate call happens LAST. */
	parm_twirler->applyRotation(90, 0.0, 1.0, 0.0);
		 /* Rotate it around the y axis so it flops down in the x axis. 
		The rotations will happen BEFORE the translate call.
		Remember that the LAST rotate call happens FIRST. */
	//Just to test out my methods, I'm going to have parm_twirler rotate.
	parm_twirler->setAnimateSpinRate(120.0); /* Default _animatespinaxis is
		z axis, which is fine as parm_twirler starts out aligned on z */
	parm_twirler->setColor(cVector(0.1, 0.1, 0.4));
	parm_twirler->setUsesTexture(false);
	add(parm_twirler);
}

void cCheerleader::animate(Real dt)
{
	/* The base class animate rotates each assembly dt*_animatespinrate around
	_animatespinaxis, and cascades the call down from composites to the
	children, eventually stopping at the polyhedra. */
	cAssemblyComposite::animate(dt);
	/* In addition, we want to alternate the top level _animatespinrate between
	positive an negative as the cheerleader reaches the _maxangle and
	_minangle. */
	if (_cumulativeanimatespinangle < _minangle)
		_animatespinrate = fabs(_animatespinrate); //Turn in + dir.
	if (_cumulativeanimatespinangle > _maxangle)
		_animatespinrate = -fabs(_animatespinrate); //Turn in - dir.
}
