/////////////////////////////////////////////////////////////////////////////
//
// texture.h : interface of the cTexture class
//
/////////////////////////////////////////////////////////////////////////////

#ifndef TEXTURE_H
#define TEXTURE_H

#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>


//======= cTextureInfo for loading files =============

//Enums and helper structures

// types of texture file formats
enum texTypes_t
{
	PCX,
	BMP
};

// only partial pcx file header
typedef struct
{
	unsigned char manufacturer;
	unsigned char version;
	unsigned char encoding;
	unsigned char bits;
	unsigned int xMin; //This and the following four lines were unsigned char
	unsigned int yMin; //And this meant I couldn't store a correct size in them.
	unsigned int xMax; //If the size was greater than 256, so a PCX of
	unsigned int yMax; //size 320 by 200 wouldn't load.
	unsigned char *palette;
} PCXHEADER;

class cTextureInfo
{
public: 
//members									
	texTypes_t textureType;
	int _width;					// width of texture
	int _height;				// height of texture
	char _skinfilename[200];
	unsigned char *_pdata;		// the texture data
	unsigned char *_ppalette;
	int _lifetime;
	bool _loaded;
//TextureInfo loading methods
	bool LoadTexture(const char *filename); //Calls one of the next two pairs
	unsigned char *LoadBitmapFile(const char *filename, BITMAPINFOHEADER *bitmapInfoHeader);
	bool LoadBMPTexture(const char *filename);
	unsigned char *LoadPCXFile(const char *filename, PCXHEADER *pcxHeader);
	bool LoadPCXTexture(const char *filename);
public:
	cTextureInfo():_pdata(NULL), _ppalette(NULL){}
	cTextureInfo(const char *filename):_pdata(NULL), _ppalette(NULL)
		{strcpy(_skinfilename, filename);LoadTexture(filename);}
	char* skinfilename(){return _skinfilename;}
	~cTextureInfo();
	bool isValid(){return _loaded;}
	unsigned char * pdata(){return _pdata;}
	void resetData(int width, int height, unsigned char *pdatanew)
		{delete [] _pdata; _pdata = pdatanew; _width = width; height = _height;}
};

//texture.h stuff=====================================
class cTexture 
{
public:
	static const double cTexture::LOG2;
	static void makePowersOfTwo(int &cx, int &cy);
protected:
	static const bool USEMIPMAP;
	static const int MAXEDGESIZE;
		/* The actual aspect of the resource image we loaded will live in double _imageaspect,
		but we will size the texture to powers of two and keep that in _cx and _cy. */ 
	bool _isvalid;
	double _imageaspect;
	int _cx; //These are powers of two that try to match the size of the resource image.
	int _cy;
	bool _transparent;
	int _pixeldataformat; //Will be GL_RGBA or GL_RGB according to _transparent TRUE or FALSE 
	int _componentsperpixel; //Will be 3 or 4 depending if GL_RGB or GB_RGBA
	bool _usemipmap;
	unsigned int _textureID; //An OpenGL-generated integer type name for a texture object.
	void _maketexture(unsigned char *pdata); /* A helper function that actually makes the
		texture.  It's called by _readDC. */
public:
	cTexture(const char *skinfilename, bool transparent = false);
	cTexture(cTextureInfo *ptextureinfo, bool transparent = false); 
	bool loadTextureInfo(cTextureInfo *ptextureinfo);
	int textureID(){return _textureID;}
	int cx(){return _cx;}
	int cy(){return _cy;}
	double imageaspect(){return _imageaspect;}
	bool usesTransparentMask(){return _transparent;}
   	void select();
		/* Makes this texture active. */
	void unselect(){::glBindTexture(GL_TEXTURE_2D, 0);} /* Makes OpenGL
		revert to the default empty texture. Also prevents texture calls from
		affecting this texture. */
	virtual ~cTexture(); //Frees up all associated resources.
};
#endif //TEXTURE_H

